/*++

Copyright (c) Sydus.

Module:

    Socket_Win32.h

Authors:

    Sharvil Nanavati (sharvil) 2006-07-30

--*/

#pragma once

#include "base.h"
#include "ISocketImplementation.h"
#include "InetAddress.h"
#include "Socket.h"

class Socket_Win32 : public ISocketImplementation
{
	public:
		explicit Socket_Win32(const ENetworkProtocol & protocol);
		~Socket_Win32();

		Socket_Win32 * Accept();
		void Bind(const InetAddress & ip, uint16 port);
		void Listen();

		void Connect(const std::string & host, uint16 port);
		void Close();

		uint32 Read(void * buffer, uint32 size) const;
		uint32 Write(const void * buffer, uint32 size) const;

		uint32 Read(InetAddress & source, void * buffer, uint32 size) const;
		uint32 Write(const InetAddress & destination, const void * buffer, uint32 size) const;

		bool IsReadable() const;
		bool IsWritable() const;

		const InetAddress & GetRemoteAddress() const;

		void SetReuseAddress(bool reuse);
		void SetBlocking();
		void SetNonBlocking();
		void EnableKeepAlive(int32 idleTime = -1, int32 interval = -1, int32 numProbes = -1);
		void DisableKeepAlive();
		void EnableNagle();
		void DisableNagle();

	private:
		Socket_Win32(SOCKET sock, const InetAddress & address);
		void InitializeWinsock() const;
		int32 GetFd() const;

		SOCKET iSocket;
		InetAddress iRemoteAddress;
};
